/*
 *  Extract all Linux logos from the Linux kernel sources (2.4.x version)
 *
 *  (C) Copyright 2000-2001 by Geert Uytterhoeven <geert@linux-m68k.org>
 *
 *  --------------------------------------------------------------------------
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of the Linux
 *  distribution for more details.
 *
 *  --------------------------------------------------------------------------
 *
 *  Compilation:
 *
 *      gcc -O3 -Wall -I$LINUX_SRC -I$LINUX_SRC/include -o logo_2_4 logo_2_4.c
 *
 */

#include <stdio.h>

#define __initdata


#define INCLUDE_LINUX_LOGO_DATA
#define INCLUDE_LINUX_LOGOBW
#define INCLUDE_LINUX_LOGO16

    /*
     *  generic
     */

#define LINUX_LOGO_COLORS 214

#include "include/linux/linux_logo.h"

#undef LINUX_LOGO_COLORS


    /*
     *  mac
     */

#define CONFIG_MAC
#define linux_logo_red mac_logo_red
#define linux_logo_green mac_logo_green
#define linux_logo_blue mac_logo_blue
#define linux_logo mac_logo
#define linux_logo_bw mac_logo_bw_unused
#define linux_logo16 mac_logo16_unused

#undef __initdata

#include "include/asm-m68k/linux_logo.h"

#undef CONFIG_MAC
#undef linux_logo_red
#undef linux_logo_green
#undef linux_logo_blue
#undef linux_logo
#undef linux_logo_bw
#undef linux_logo16
#undef linux_logo_banner
#undef LINUX_LOGO_COLORS


    /*
     *  mips
     */

#define linux_logo_red mips_logo_red
#define linux_logo_green mips_logo_green
#define linux_logo_blue mips_logo_blue
#define linux_logo mips_logo
#define linux_logo_bw mips_logo_bw_unused
#define linux_logo16 mips_logo16_unused

#include "include/asm-mips/linux_logo.h"

#undef linux_logo_red
#undef linux_logo_green
#undef linux_logo_blue
#undef linux_logo
#undef linux_logo_bw
#undef linux_logo16
#undef linux_logo_banner
#undef LINUX_LOGO_COLORS


    /*
     *  mips64
     */

#define linux_logo_red mips64_logo_red
#define linux_logo_green mips64_logo_green
#define linux_logo_blue mips64_logo_blue
#define linux_logo mips64_logo
#define linux_logo_bw mips64_logo_bw_unused
#define linux_logo16 mips64_logo16_unused

#include "include/asm-mips64/linux_logo.h"

#undef linux_logo_red
#undef linux_logo_green
#undef linux_logo_blue
#undef linux_logo
#undef linux_logo_bw
#undef linux_logo16
#undef linux_logo_banner
#undef LINUX_LOGO_COLORS


    /*
     *  sparc
     */

#define linux_logo_red sparc_logo_red
#define linux_logo_green sparc_logo_green
#define linux_logo_blue sparc_logo_blue
#define linux_logo sparc_logo
#define linux_logo_bw sparc_logo_bw_unused
#define linux_logo16 sparc_logo16_unused
#undef INCLUDE_LINUX_LOGO16

#include "include/asm-sparc/linux_logo.h"

#undef linux_logo_red
#undef linux_logo_green
#undef linux_logo_blue
#undef linux_logo
#undef linux_logo_bw
#undef linux_logo16
#undef linux_logo_banner
#undef LINUX_LOGO_COLORS
#define INCLUDE_LINUX_LOGO16


    /*
     *  sparc64
     */

#define linux_logo_red sparc64_logo_red
#define linux_logo_green sparc64_logo_green
#define linux_logo_blue sparc64_logo_blue
#define linux_logo sparc64_logo
#define linux_logo_bw sparc64_logo_bw_unused
#define linux_logo16 sparc64_logo16_unused
#undef INCLUDE_LINUX_LOGO16

#include "include/asm-sparc64/linux_logo.h"

#undef linux_logo_red
#undef linux_logo_green
#undef linux_logo_blue
#undef linux_logo
#undef linux_logo_bw
#undef linux_logo16
#undef linux_logo_banner
#undef LINUX_LOGO_COLORS
#define INCLUDE_LINUX_LOGO16


    
static void write_logo(const char *filename, const unsigned char *data,
		       const unsigned char *red, const unsigned char *green,
		       const unsigned char *blue)
{
    FILE *stream;
    int i, j, d;
    
    stream = fopen(filename, "w");
    if (!stream) {
	perror("file open error: ");
	exit(1);
    }
    fputs("P3\n80 80\n255\n", stream);
    for (i = 0; i < 80*80; i += 4) {
	for (j = 0; j < 4; j++) {
	    d = data[i+j]-32;
	    fprintf(stream, " %3d %3d %3d", red[d], green[d], blue[d]);
	}
	fputc('\n', stream);
    }
    fclose(stream);
}

static void write_logo_bw(const char *filename, const unsigned char *data)
{
    FILE *stream;
    int i, j, d;
    
    stream = fopen(filename, "w");
    if (!stream) {
	perror("file open error: ");
	exit(1);
    }
    fputs("P1\n80 80\n", stream);
    for (i = 0; i < 80*80/8; i += 4) {
	for (j = 0; j < 4; j++) {
	    d = data[i+j];
	    fprintf(stream, " %1d %1d %1d %1d %1d %1d %1d %1d",
		    (d >> 7) & 1, (d >> 6) & 1, (d >> 5) & 1, (d >> 4) & 1,
		    (d >> 3) & 1, (d >> 2) & 1, (d >> 1) & 1, d & 1);
	}
	fputc('\n', stream);
    }
    fclose(stream);
}

static struct entry {
    unsigned char red;
    unsigned char green;
    unsigned char blue;
} palette16[16] = {
    { 0, 0, 0, },
    { 0, 0, 170, },
    { 0, 170, 0, },
    { 0, 170, 170, },
    { 170, 0, 0, },
    { 170, 0, 170, },
    { 170, 85, 0, },
    { 170, 170, 170, },
    { 85, 85, 85, },
    { 85, 85, 255, },
    { 85, 255, 85, },
    { 85, 255, 255, },
    { 255, 85, 85, },
    { 255, 85, 255, },
    { 255, 255, 85, },
    { 255, 255, 255 },
};

static void write_logo16(const char *filename, const unsigned char *data)
{
    FILE *stream;
    int i, j, d;
    
    stream = fopen(filename, "w");
    if (!stream) {
	perror("file open error: ");
	exit(1);
    }
    fputs("P3\n80 80\n255\n", stream);
    for (i = 0; i < 80*80/2; i += 2) {
	for (j = 0; j < 2; j++) {
	    d = data[i+j] >> 4;
	    fprintf(stream, " %3d %3d %3d", palette16[d].red,
		    palette16[d].green, palette16[d].blue);
	    d = data[i+j] & 15;
	    fprintf(stream, " %3d %3d %3d", palette16[d].red,
		    palette16[d].green, palette16[d].blue);
	}
	fputc('\n', stream);
    }
    fclose(stream);
}

int main(int argc, char *argv[])
{
    /* generic */
    write_logo("logo_2_4.ppm", linux_logo, linux_logo_red, linux_logo_green,
    	       linux_logo_blue);
    write_logo_bw("logo_bw_2_4.pbm", linux_logo_bw);
    write_logo16("logo16_2_4.ppm", linux_logo16);
    /* mac */
    write_logo("mac_2_4.ppm", mac_logo, mac_logo_red, mac_logo_green,
    	       mac_logo_blue);
    /* mips */
    write_logo("mips_2_4.ppm", mips_logo, mips_logo_red, mips_logo_green,
    	       mips_logo_blue);
    /* mips64 */
    write_logo("mips64_2_4.ppm", mips64_logo, mips64_logo_red,
	       mips64_logo_green, mips64_logo_blue);
    /* sparc */
    write_logo("sparc_2_4.ppm", sparc_logo, sparc_logo_red, sparc_logo_green,
    	       sparc_logo_blue);
    /* sparc64 */
    write_logo("sparc64_2_4.ppm", sparc64_logo, sparc64_logo_red,
	       sparc64_logo_green, sparc64_logo_blue);
    return 0;
}

